@@ -254,8 +254,8 @@ CURLcode Curl_rtsp(struct connectdata *conn, bool *done)
   }
 
   if(rtspreq == RTSPREQ_RECEIVE) {
-    result = Curl_setup_transfer(conn, FIRSTSOCKET, -1, TRUE,
-                                 &http->readbytecount, -1, NULL);
+    Curl_setup_transfer(conn, FIRSTSOCKET, -1, TRUE,
+                        &http->readbytecount, -1, NULL);
 
     return result;
   }
@@ -503,15 +503,9 @@ CURLcode Curl_rtsp(struct connectdata *conn, bool *done)
     return result;
   }
 
-  result = Curl_setup_transfer(conn, FIRSTSOCKET, -1, TRUE,
-                               &http->readbytecount,
-                               putsize?FIRSTSOCKET:-1,
-                               putsize?&http->writebytecount:NULL);
-
-  if(result) {
-    failf(data, "Failed RTSP transfer");
-    return result;
-  }
+  Curl_setup_transfer(conn, FIRSTSOCKET, -1, TRUE, &http->readbytecount,
+                      putsize?FIRSTSOCKET:-1,
+                      putsize?&http->writebytecount:NULL);
 
   /* Increment the CSeq on success */
   data->state.rtsp_next_client_CSeq++;
@@ -715,7 +709,7 @@ CURLcode Curl_rtsp_parseheader(struct connectdata *conn,
     while(*start && ISSPACE(*start))
       start++;
 
-    if(!start) {
+    if(!*start) {
       failf(data, "Got a blank Session ID");
     }
     else if(data->set.str[STRING_RTSP_SESSION_ID]) {
