@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2010, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,6 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: urlglob.c,v 1.54 2009-06-05 16:14:50 yangtse Exp $
  ***************************************************************************/
 
 /* client-local setup.h */
@@ -76,8 +75,6 @@ static GlobCode glob_set(URLGlob *glob, char *pattern,
   ++glob->size;
 
   while (!done) {
-    bool skip;
-
     switch (*pattern) {
     case '\0':                  /* URL ended while set was still open */
       snprintf(glob->errormsg, sizeof(glob->errormsg),
@@ -110,7 +107,7 @@ static GlobCode glob_set(URLGlob *glob, char *pattern,
 
         /* always check for a literal (may be "") between patterns */
         if(GLOB_ERROR == glob_word(glob, ++pattern, ++pos, &wordamount))
-          wordamount=1;
+          return GLOB_ERROR;
         *amount = pat->content.Set.size * wordamount;
 
         done = TRUE;
@@ -128,24 +125,7 @@ static GlobCode glob_set(URLGlob *glob, char *pattern,
       return GLOB_ERROR;
 
     case '\\':                          /* escaped character, skip '\' */
-      switch(pattern[1]) {
-      case '[':
-      case ']':
-      case '{':
-      case '}':
-      case ',':
-        skip = TRUE;
-        break;
-      default:
-        skip = FALSE;
-        break;
-      }
-      if(skip) {
-        if (*(buf+1) == '\0') {           /* but no escaping of '\0'! */
-          snprintf(glob->errormsg, sizeof(glob->errormsg),
-                   "illegal pattern at pos %zu\n", pos);
-          return GLOB_ERROR;
-        }
+      if(pattern[1]) {
         ++pattern;
         ++pos;
       }
@@ -282,8 +262,11 @@ static GlobCode glob_word(URLGlob *glob, char *pattern,
   *amount = 1; /* default is one single string */
 
   while (*pattern != '\0' && *pattern != '{' && *pattern != '[') {
-    if (*pattern == '}' || *pattern == ']')
+    if (*pattern == '}' || *pattern == ']') {
+      snprintf(glob->errormsg, sizeof(glob->errormsg),
+               "unmatched close brace/bracket at pos %zu\n", pos);
       return GLOB_ERROR;
+    }
 
     /* only allow \ to escape known "special letters" */
     if (*pattern == '\\' &&
@@ -293,8 +276,6 @@ static GlobCode glob_word(URLGlob *glob, char *pattern,
       /* escape character, skip '\' */
       ++pattern;
       ++pos;
-      if (*pattern == '\0')             /* but no escaping of '\0'! */
-        return GLOB_ERROR;
     }
     *buf++ = *pattern++;                /* copy character to literal */
     ++pos;
@@ -343,7 +324,7 @@ int glob_url(URLGlob** glob, char* url, int *urlnum, FILE *error)
   if(NULL == glob_buffer)
     return CURLE_OUT_OF_MEMORY;
 
-  glob_expand = calloc(sizeof(URLGlob), 1);
+  glob_expand = calloc(1, sizeof(URLGlob));
   if(NULL == glob_expand) {
     free(glob_buffer);
     return CURLE_OUT_OF_MEMORY;
@@ -357,7 +338,7 @@ int glob_url(URLGlob** glob, char* url, int *urlnum, FILE *error)
   else {
     if(error && glob_expand->errormsg[0]) {
       /* send error description to the error-stream */
-      fprintf(error, "curl: (%d) [globbing] %s\n",
+      fprintf(error, "curl: (%d) [globbing] %s",
               CURLE_URL_MALFORMAT, glob_expand->errormsg);
     }
     /* it failed, we cleanup */
